NUM_FPGAS = 4
DDR_PER_FPGA = 2
FPGA_IO_PATTERN = r'^io_'
FPGA_IO_DIFF_PATTERN = r'^io_l\d+p'
FPGA_TXP_PATTERN = r'^mgttxp'
FPGA_RXP_PATTERN = r'^mgtrxp'
DDR_IO_PATTERNS = tuple(
    re.compile(R, flags=re.IGNORECASE) for R in (
        r'^(a|ba|cke|dm|dq|odt|sa)\d+$',
        r'^(cas|cs\d|ras|we)_n$',
        r'^(scl|sda)$',
    )
)
DDR_IO_DIFF_PATTERNS = tuple(
    re.compile(R, flags=re.IGNORECASE) for R in (
        r'^(ck|dqs)\d+$',
    )
)
PCIE_TXP_PATTERN = r'^hso\d_p$'
PCIE_RXP_PATTERN = r'^hsi\d_p$'
FPGA_RING_WIDTH = 36    # Differential!
PCIE_WIDTH = 8
PCIE_SEP_MM = 12    # Center-center spacing in millimeters

middle_index = (NUM_FPGAS - 1) / 2

for i in range(NUM_FPGAS):
    FPGA_DIN = AddDevice(
        'FPGA{}'.format(i),
        'U', ['0 mm', '0 mm', '0 mm', '0 deg'],
        'fpga/xilinx/xc5vfx100t', 'ff1136', 'ffg1136', 'direct'
    )

    pcie_x = (i - middle_index) * PCIE_SEP_MM
    pcie_DIN = AddDevice(
        'PCIE{}'.format(i),
        'P', ['{} mm'.format(pcie_x), '0 mm', '0 mm', '90 deg'],
        'connectors/pcie', 'x8', 'pcie8x', 'direct'
    )

    # Lock PCIe connector in place
    S.locked_DINs.add(pcie_DIN)

    for tag, pattern in (
        ('TX', FPGA_TXP_PATTERN),
        ('RX', FPGA_RXP_PATTERN),
    ):
        AddSimpleEP(
            'Bundle_PCIE{}{}'.format(tag, i), 'FPGA{}'.format(i),
            pinfunc_match, [pattern], mf_kwargs={
                'ignorecase': True,
            }
        )

        BundleNets(
            'Bundle_PCIE{}{}'.format(tag, i),
            tuple(
                'N_PCIE{}{}<{}>'.format(tag, i, v)
                for v in range(PCIE_WIDTH)
            )
        )

        BundleSetDifferential(
            'Bundle_PCIE{}{}'.format(tag, i),
            '_P',
            '_N'
        )

    for tag, pattern in (
        ('TX', PCIE_TXP_PATTERN),
        ('RX', PCIE_RXP_PATTERN),
    ):
        AddSimpleEP(
            'Bundle_PCIE{}{}'.format(tag, i), 'PCIE{}'.format(i),
            pinfunc_match, [pattern], mf_kwargs={
                'ignorecase': True,
            }
        )


for i in range(NUM_FPGAS):
    for j in range(DDR_PER_FPGA):
        DDR_DIN = AddDevice('DDR{}.{}'.format(i, j),
                  'U', ['0 mm', '0 mm', '0 mm', '0 deg'],
                  'ram/ddr2', 'dimm240', 'dimm240_87705', 'direct')

def match_any_re(devinst, logpinkey, logpin, re_iterable):
    """Return True if any pinfunc matches any compiled RE in re_iterable."""

    for k, pf in logpin.pinfuncs.items():
        for R in re_iterable:
            if R.match(k):
                return True

    return False


# DEBUG: print number of eligible FPGA pins
FPGA_IO_PINS = 0
FPGA_IO_DIFF_PINS = 0
DI = S.devinst_n2o[FPGA_DIN]
for lpk, logpin in DI.logpack.pins.items():
    if pinfunc_match(DI, lpk, logpin, FPGA_IO_PATTERN, ignorecase=True):
        FPGA_IO_PINS += 1
    if pinfunc_match(DI, lpk, logpin, FPGA_IO_DIFF_PATTERN, ignorecase=True):
        FPGA_IO_DIFF_PINS += 1
del DI

print('# of eligible FPGA IO pins:', FPGA_IO_PINS)
print('# of eligible FPGA differential pairs:', FPGA_IO_DIFF_PINS)


# Compute number of nets required to connect to each DDR
DDR_NUM_NETS = 0
DI = S.devinst_n2o[DDR_DIN]
for lpk, logpin in DI.logpack.pins.items():
    if match_any_re(DI, lpk, logpin, DDR_IO_PATTERNS):
        DDR_NUM_NETS += 1
del DI

DDR_NUM_DIFF_NETS = 0
DI = S.devinst_n2o[DDR_DIN]
for lpk, logpin in DI.logpack.pins.items():
    if match_any_re(DI, lpk, logpin, DDR_IO_DIFF_PATTERNS):
        DDR_NUM_DIFF_NETS += 1
del DI

print('Require', DDR_NUM_NETS, 'single-ended nets')
print('Require', DDR_NUM_DIFF_NETS, 'differential pairs')


for i in range(NUM_FPGAS):
    for j in range(DDR_PER_FPGA):

        ### Single-ended nets
        AddSimpleEP(
            'Bundle_DDR{}.{}'.format(i, j), 'FPGA{}'.format(i),
            pinfunc_match, [FPGA_IO_PATTERN], mf_kwargs={
                'ignorecase': True,
            }
        )

        AddSimpleEP(
            'Bundle_DDR{}.{}'.format(i, j), 'DDR{}.{}'.format(i, j),
            match_any_re, [DDR_IO_PATTERNS]
        )

        BundleNets(
            'Bundle_DDR{}.{}'.format(i, j),
            tuple(
                'N_DDR{}.{}<{}>'.format(i, j, v)
                for v in range(DDR_NUM_NETS)
            )
        )

        ### Differential nets
        AddSimpleEP(
            'Bundle_DDR_diff{}.{}'.format(i, j), 'FPGA{}'.format(i),
            pinfunc_match, [FPGA_IO_DIFF_PATTERN], mf_kwargs={
                'ignorecase': True,
            }
        )

        AddSimpleEP(
            'Bundle_DDR_diff{}.{}'.format(i, j), 'DDR{}.{}'.format(i, j),
            match_any_re, [DDR_IO_DIFF_PATTERNS]
        )

        BundleNets(
            'Bundle_DDR_diff{}.{}'.format(i, j),
            tuple(
                'N_DDR_diff{}.{}<{}>'.format(i, j, v)
                for v in range(DDR_NUM_DIFF_NETS)
            )
        )

        BundleSetDifferential(
            'Bundle_DDR_diff{}.{}'.format(i, j),
            '_P',
            '_N'
        )



fpga_pairs = [(i, i+1) for i in range(NUM_FPGAS-1)]
fpga_pairs.append((NUM_FPGAS-1, 0))

for n1, n2 in fpga_pairs:
    # Add ring connection between these two FPGAs
    bundlename = 'B_RING_{}_{}'.format(n1, n2)

    AddSimpleEP(
        bundlename, 'FPGA{}'.format(n1),
        pinfunc_match, [FPGA_IO_DIFF_PATTERN], mf_kwargs={
            'ignorecase': True,
        }
    )

    AddSimpleEP(
        bundlename, 'FPGA{}'.format(n2),
        pinfunc_match, [FPGA_IO_DIFF_PATTERN], mf_kwargs={
            'ignorecase': True,
        }
    )

    BundleNets(
        bundlename,
        tuple(
            'N_RING_{}_{}<{}>'.format(n1, n2, i)
            for i in range(FPGA_RING_WIDTH)
        )
    )

    BundleSetDifferential(bundlename, '_P', '_N')
