### Based on the CMS Calorimeter Trigger Processor board [Klabbers, et al]

FPGA_IO_PATTERN = r'^io_'
FPGA_IO_DIFF_PATTERN = r'^io_l\d+p'
FPGA_TXP_PATTERN = r'^mgttxp'
FPGA_RXP_PATTERN = r'^mgtrxp'
DDR_IO_PATTERNS = tuple(
    re.compile(R, flags=re.IGNORECASE) for R in (
        r'^(a|ba|dq)\d+$',
        r'^(cke|[ul]dm|odt)$',
        r'^(cas|cs|ras|we|reset|s\d)_n$',
    )
)
DDR_IO_DIFF_PATTERNS = tuple(
    re.compile(R, flags=re.IGNORECASE) for R in (
        r'^(ck|[ul]dqs)+$',
    )
)

FPGA_FLASH_PATTERNS = (
    ('A', r'^a\d+$', r'^a(\d|1\d|2[0-6])$'),
    ('D', r'^d\d+$', r'^dq\d+$'),
    #('CS', 'fcs_b', 'ce_n'),
    #('OE', 'foe_b', 'oe_n'),
    #('WE', 'fwe_b', 'we_n'),
)

coords_dontcare = ['0 mm', '0 mm', '0 mm', '0 deg']

AddDevice(
    'FPGA_F',
    'U', coords_dontcare,
    'fpga/xilinx/xc6vhx250t', 'ff1154', 'ffg1157', 'direct'
)
AddDevice(
    'FPGA_B',
    'U', coords_dontcare,
    'fpga/xilinx/xc6vhx250t', 'ff1154', 'ffg1157', 'direct'
)

FLASH_DIN = AddDevice(
    'Flash',
    'U', coords_dontcare,
    'flash/p30', 'bga64', 'bga64', 'direct'
)

DDR_DIN = AddDevice(
    'DDR_F',
    'U', coords_dontcare,
    'ram/mt41j128m16', 'bga96', 'bga96', 'direct'
)
AddDevice(
    'DDR_B',
    'U', coords_dontcare,
    'ram/mt41j128m16', 'bga96', 'bga96', 'direct'
)


def match_any_re(devinst, logpinkey, logpin, re_iterable):
    """Return True if any pinfunc matches any compiled RE in re_iterable."""

    for k, pf in logpin.pinfuncs.items():
        for R in re_iterable:
            if R.match(k):
                return True

    return False


### FPGA <-> Flash connections
DI = S.devinst_n2o[FLASH_DIN]
for abbr, FPGA_pat, Flash_pat in FPGA_FLASH_PATTERNS:
    bundlename = 'Bundle_Flash_' + abbr

    ### Detect how many nets are required.
    numnets = 0

    for lpk, logpin in DI.logpack.pins.items():
        if pinfunc_match(DI, lpk, logpin, Flash_pat, ignorecase=True):
            numnets += 1

    if numnets < 1:
        raise ValueError('Not enough pins!', abbr, Flash_pat)
    elif numnets == 1:
        netnames = ['F_' + abbr]
    else:
        netnames = bus2netnames('F_{}<'.format(abbr), '>', range(numnets))

    AddSimpleEP(
        bundlename, 'FPGA_B',
        pinfunc_match, [FPGA_pat], mf_kwargs={'ignorecase': True},
    )

    AddSimpleEP(
        bundlename, 'Flash',
        pinfunc_match, [Flash_pat], mf_kwargs={'ignorecase': True},
    )

    BundleNets(bundlename, netnames)

    BundleParams(bundlename, dict(iostandard='lvcmos25'))

    print(bundlename, 'requires', numnets)


DI = S.devinst_n2o[DDR_DIN]
for side in ('F', 'B'):
    FPGA = 'FPGA_' + side
    DDR = 'DDR_' + side

    ### Detect how many nets are required.
    numnets_single = 0
    numnets_diff = 0

    for lpk, logpin in DI.logpack.pins.items():
        if match_any_re(DI, lpk, logpin, DDR_IO_PATTERNS):
            numnets_single += 1
        if match_any_re(DI, lpk, logpin, DDR_IO_DIFF_PATTERNS):
            numnets_diff += 1

    netnames_single = bus2netnames(
        'DDR_{}_s<'.format(side),
        '>',
        range(numnets_single)
    )

    netnames_diff = bus2netnames(
        'DDR_{}_d<'.format(side),
        '>',
        range(numnets_diff)
    )


    bundlename = 'Bundle_DDR_' + side

    AddSimpleEP(
        bundlename, FPGA,
        pinfunc_match, [FPGA_IO_PATTERN],
        mf_kwargs={'ignorecase': True}
    )

    AddSimpleEP(
        bundlename, DDR,
        match_any_re, [DDR_IO_PATTERNS]
    )

    BundleNets(bundlename, netnames_single)

    BundleParams(bundlename, dict(iostandard='lvcmos15'))

    print(bundlename, 'requires', numnets_single)


    bundlename = 'Bundle_DDR_' + side + '_diff'

    AddSimpleEP(
        bundlename, FPGA,
        pinfunc_match, [FPGA_IO_DIFF_PATTERN],
        mf_kwargs={'ignorecase': True}
    )

    AddSimpleEP(
        bundlename, DDR,
        match_any_re, [DDR_IO_DIFF_PATTERNS]
    )

    BundleNets(bundlename, netnames_diff)

    BundleParams(bundlename, dict(iostandard='lvdci_15'))

    BundleSetDifferential(bundlename, '_P', '_N')

    print(bundlename, 'requires', numnets_diff)


### F <-> B parallel connections.
numnets = 12
netnames = bus2netnames('PAR<', '>', range(numnets))

bundlename = 'Bundle_PAR'

AddSimpleEP(
    bundlename, 'FPGA_F',
    pinfunc_match, [FPGA_IO_PATTERN],
    mf_kwargs={'ignorecase': True}
)

AddSimpleEP(
    bundlename, 'FPGA_B',
    pinfunc_match, [FPGA_IO_PATTERN],
    mf_kwargs={'ignorecase': True}
)

BundleNets(bundlename, netnames)

BundleParams(bundlename, dict(iostandard='lvcmos25'))

print(bundlename, 'requires', numnets)


### F -> B forwarding links
numnets = 24
netnames = bus2netnames('FWD<', '>', range(numnets))

bundlename = 'Bundle_FWD'

AddSimpleEP(
    bundlename, 'FPGA_F',
    pinfunc_match, [FPGA_TXP_PATTERN],
    mf_kwargs={'ignorecase': True}
)

AddSimpleEP(
    bundlename, 'FPGA_B',
    pinfunc_match, [FPGA_RXP_PATTERN],
    mf_kwargs={'ignorecase': True}
)

BundleNets(bundlename, netnames)

BundleSetDifferential(bundlename, '_P', '_N')

print(bundlename, 'requires', numnets)
