dontcare = '0 mm,0 mm,0 mm,0 deg'.split(',')

FPGA_IO_PATTERN = 'IO_'
USB_PATTERNS = (r'ADBUS\d', r'ACBUS\d', r'BDBUS\d', r'BCBUS\d')
USB_BUSNAMES = ('AD', 'AC', 'BD', 'BC')
DRV_CTLIN_PATTERN = r'(SIN|CLK|LATCH|BLANK)'

AddDevice('FPGA', 'U', dontcare,
          'fpga/xilinx/xc3s200a', 'vq100', 'vqfp100', 'direct')

AddDevice('USB', 'U', dontcare, 'usb/ft2232h', 'L', 'lqfp64', 'direct')

for pat, bus in zip(USB_PATTERNS, USB_BUSNAMES):
    # Add series resistors.
    r = 'R_' + bus
    AddDevice(r, 'R', dontcare, 'passive/r8', 'SO-16', '0603x8', 'direct')

    # Bundle from USB IC to resistors.
    bundlename = 'USB2R_' + bus
    netnames = bus2netnames(bundlename + '[', ']', range(8))

    AddSimpleEP(bundlename, 'USB', pinfunc_match, [pat])
    AddSimpleEP(bundlename, r, pinfunc_match, [r'A\d'])
    BundleNets(bundlename, netnames)

    # Bundle from FPGA to resistors.
    bundlename = 'R2FPGA_' + bus
    netnames = bus2netnames(bundlename + '[', ']', range(8))

    AddSimpleEP(bundlename, 'FPGA', pinfunc_match, [FPGA_IO_PATTERN])
    AddSimpleEP(bundlename, r, pinfunc_match, [r'B\d'])
    BundleNets(bundlename, netnames)

AddDevice('XTAL', 'X', dontcare, 'passive/xtal', '2pin', 'hc49us', 'direct')
for i in ('1', '2'):
    netname = 'FTDI_XTAL' + i
    bundlename = 'B_' + netname
    BundleNets(bundlename, [netname])
    AddSimpleEP(bundlename, 'USB', pinfunc_match, ['OSC.'])
    AddSimpleEP(bundlename, 'XTAL', has_pinfunc, [i])

# Logic-level translator
AddDevice('TRANS', 'U', dontcare, 'logic/74vhct245a', 'SO-20', 'so20_w', 'direct')
bundlename = 'FPGA2TRANS'
netnames = bus2netnames(bundlename + '[', ']', range(8))

AddSimpleEP(bundlename, 'FPGA', pinfunc_match, [FPGA_IO_PATTERN])
AddSimpleEP(bundlename, 'TRANS', in_bank, ['A'])
BundleNets(bundlename, netnames)
BundleParams(bundlename, dict(iostandard='lvcmos33'))

# LED drivers
AddDevice('DRV1', 'U', dontcare, 'driver/cat4016', 'SO-24', 'so24_w', 'direct')
AddDevice('DRV2', 'U', dontcare, 'driver/cat4016', 'SO-24', 'so24_w', 'direct')
bundlename = 'DRVCTL'
netnames = bus2netnames(bundlename + '[', ']', range(8))

AddSimpleEP(bundlename, 'TRANS', in_bank, ['B'])
AddMultigroupEP(bundlename, (
    (AUTO, 'DRV' + i, pinfunc_match, [DRV_CTLIN_PATTERN], None, None)
    for i in ('1', '2')
))
BundleNets(bundlename, netnames)
BundleParams(bundlename, dict(iostandard='cmos'))

# LED connectors
for i in ('1', '2'):
    for bank in ('L', 'H'):
        AddDevice('LEDC' + i + bank, 'P', dontcare, 'connectors/conn8', 'sip8', 'sip8', 'direct')
        bundlename = 'LED' + i + bank
        netnames = bus2netnames(bundlename + '[', ']', range(8))

        AddSimpleEP(bundlename, 'DRV' + i, in_bank, ['LED' + bank])
        AddSimpleEP(bundlename, 'LEDC' + i + bank, any_pin, [])
        BundleNets(bundlename, netnames)
