
WIDTH = 32  # Width of each inter-FPGA bus
IO_PATTERN = r'^io_'

GRID_COLS = set(range(3))
GRID_ROWS = set(range(3))


def grid2nickname(col, row):
    return 'X{:}Y{:}'.format(col, row)

def grid2coords(col, row):
    x = col * 50
    y = row * 50
    z = 0
    coords = ['{:} mm'.format(v) for v in (x, y, z)]
    coords.append('0 deg')
    return coords


### Add devices in grid
for col in GRID_COLS:
    for row in GRID_ROWS:
        nickname = grid2nickname(col, row)
        coords = grid2coords(col, row)

        AddDevice(
            nickname, 'U', coords,
            'fpga/xilinx/xc7vx485t', 'ffg1157', 'ffg1157', 'direct'
        )


def devs2bundlename(nickname_1, nickname_2):
    return '_'.join(('B', nickname_1, nickname_2))

def devs2netnames(nickname_1, nickname_2, numnets):
    return [
        '{:}_{:}_d<{:}>'.format(
            nickname_1,
            nickname_2,
            i
        )
        for i in range(numnets)
    ]

def relcoords2dir(d_col, d_row):
    if d_col < 0:
        d_col = 'n'
    elif d_col > 0:
        d_col = 'p'
    else:
        d_col = 'z'

    if d_row < 0:
        d_row = 'n'
    elif d_row > 0:
        d_row = 'p'
    else:
        d_row = 'z'

    return {
        'zn': 'N',
        'zp': 'S',
        'nz': 'W',
        'pz': 'E',
    }.get(d_col + d_row, 'N')


def match_re_and_set(devinst, logpinkey, logpin,
                     pinfunc_re, ignorecase=False,
                     allow_set=frozenset()):

    if logpinkey not in allow_set:
        return False

    flags = 0
    if ignorecase:
        flags |= re.IGNORECASE

    R = re.compile(pinfunc_re, flags=flags)

    for k, pf in logpin.pinfuncs.items():
        if R.match(k):
            return True

    return False


dir2allowset = {
    'N': ('ABCDEF', range(1, 34)),
    'S': ('AJ AK AL AM AN AP'.split(), range(1, 34)),
    'W': ('A B C D E F G H J K L M AC AD AE AF AG AH AJ AK AL AM AN AP'.split(), range(8, 15)),
    'E': ('A B C D E F G H J K L M AC AD AE AF AG AH AJ AK AL AM AN AP'.split(), range(28, 34)),
}

for d, allowset in dir2allowset.items():
    dir2allowset[d] = frozenset(
        ''.join(map(str, i)) for i in itertools.product(*allowset)
    )


### Add Bundle connections between devices
for col_1 in GRID_COLS:
    for row_1 in GRID_ROWS:
        nickname_1 = grid2nickname(col_1, row_1)

        # Connect to device EAST, device SOUTH
        for d_col, d_row in ((1, 0), (0, 1)):
            col_2 = col_1 + d_col
            row_2 = row_1 + d_row

            if col_2 not in GRID_COLS:
                continue
            if row_2 not in GRID_ROWS:
                continue

            dir_1 = relcoords2dir(d_col, d_row)
            dir_2 = relcoords2dir(- d_col, - d_row)

            nickname_2 = grid2nickname(col_2, row_2)

            bundlename = devs2bundlename(nickname_1, nickname_2)
            netnames = devs2netnames(nickname_1, nickname_2, WIDTH)

            BundleNets(bundlename, netnames)

            AddSimpleEP(
                bundlename, nickname_1,
                match_re_and_set, [IO_PATTERN], mf_kwargs={
                    'ignorecase': True,
                    'allow_set': dir2allowset[dir_1],
                }
            )

            AddSimpleEP(
                bundlename, nickname_2,
                match_re_and_set, [IO_PATTERN], mf_kwargs={
                    'ignorecase': True,
                    'allow_set': dir2allowset[dir_2],
                }
            )

