import random

WIDTH = 48  # Width of each inter-FPGA bus
IO_PATTERN = r'^io_'

GRID_COLS = set(range(3))
GRID_ROWS = set(range(3))

exclude = {(0,0), (0,2), (2,0), (2,2)}

coords_zero = ['0 mm', '0 mm', '0 mm', '0 deg']


def grid2nickname(col, row):
    return 'X{:}Y{:}'.format(col, row)

### Add devices in grid
for col in GRID_COLS:
    for row in GRID_ROWS:
        if (col, row) in exclude:
            continue

        nickname = grid2nickname(col, row)

        DIN = AddDevice(
            nickname, 'U', coords_zero,
            'fpga/xilinx/xc7vx485t', 'ffg1157', 'ffg1157', 'direct'
        )

        if (col, row) == (1, 1):
            S.locked_DINs.add(DIN)


def devs2bundlename(nickname_1, nickname_2):
    return '_'.join(('B', nickname_1, nickname_2))

def devs2netnames(nickname_1, nickname_2, numnets):
    return [
        '{:}_{:}_d<{:}>'.format(
            nickname_1,
            nickname_2,
            i
        )
        for i in range(numnets)
    ]

def relcoords2dir(d_col, d_row):
    if d_col < 0:
        d_col = 'n'
    elif d_col > 0:
        d_col = 'p'
    else:
        d_col = 'z'

    if d_row < 0:
        d_row = 'n'
    elif d_row > 0:
        d_row = 'p'
    else:
        d_row = 'z'

    return {
        'zn': 'N',
        'zp': 'S',
        'nz': 'W',
        'pz': 'E',
    }.get(d_col + d_row, 'N')


# dir2bank: give a bank number near the N/E/S/W edge
dir2bank = {
    'N': 37,
    'S': 36,
    'W': 39,
    'E': 18,
}


### Add Bundle connections between devices
for col_1 in GRID_COLS:
    for row_1 in GRID_ROWS:
        if (col_1, row_1) in exclude:
            continue

        nickname_1 = grid2nickname(col_1, row_1)

        # Connect to device EAST, device SOUTH
        for d_col, d_row in ((1, 0), (0, 1)):
            col_2 = col_1 + d_col
            row_2 = row_1 + d_row

            if col_2 not in GRID_COLS:
                continue
            if row_2 not in GRID_ROWS:
                continue

            if (col_2, row_2) in exclude:
                continue

            dir_1 = relcoords2dir(d_col, d_row)
            dir_2 = relcoords2dir(- d_col, - d_row)

            nickname_2 = grid2nickname(col_2, row_2)

            bundlename = devs2bundlename(nickname_1, nickname_2)
            netnames = devs2netnames(nickname_1, nickname_2, WIDTH)

            BundleNets(bundlename, netnames)

            AddSimpleEP(
                bundlename, nickname_1,
                pinfunc_match, [IO_PATTERN], mf_kwargs={
                    'ignorecase': True,
                }
            )

            AddSimpleEP(
                bundlename, nickname_2,
                pinfunc_match, [IO_PATTERN], mf_kwargs={
                    'ignorecase': True,
                }
            )

