import itertools

WIDTH = 8   # Width of inter-FPGA bus
IO_PATTERN = r'^io_'

coords = '0 mm,0 mm,0 mm,0 deg'.split(',')

### Want FPGAs to be named U3 and U4 to match paper example
### Consume U1 and U2 names
S.next_devinstname('U') # U1
S.next_devinstname('U') # U2

# U3
AddDevice(
    'FPGA1', 'U', coords,
    'fpga/xilinx/xc7vx485t', 'ffg1157', 'ffg1157', 'direct'
)

# U4
AddDevice(
    'FPGA2', 'U', coords,
    'fpga/xilinx/xc7vx485t', 'ffg1157', 'ffg1157', 'direct'
)

# AC-coupling caps
for i in range(WIDTH):
    AddDevice(
        'Cap{}'.format(i), 'C', coords,
        'passive/c', 'nonpolar', '0402', 'direct'
    )


### Add Bundle between FPGA1 and caps
netnames = tuple('A{:}'.format(i) for i in range(WIDTH))
BundleNets('FPGA1_Data', netnames)

AddSimpleEP(
    'FPGA1_Data', 'FPGA1',
    pinfunc_match, [IO_PATTERN], mf_kwargs={
        'ignorecase': True,
    }
)

AddMultigroupEP('FPGA1_Data', (
    (AUTO, 'Cap{}'.format(i), any_pin, None, None, None)
    for i in range(WIDTH)
))



### Add Bundle between FPGA2 and caps
netnames = tuple('B{:}'.format(i) for i in range(WIDTH))
BundleNets('FPGA2_Data', netnames)

AddSimpleEP(
    'FPGA2_Data', 'FPGA2',
    pinfunc_match, [IO_PATTERN], mf_kwargs={
        'ignorecase': True,
    }
)

AddMultigroupEP('FPGA2_Data', (
    (AUTO, 'Cap{}'.format(i), any_pin, None, None, None)
    for i in range(WIDTH)
))


### Define user cost function for debugging.
def user_cost_func(placement, state, it):
    iteration = next(it)
    return float(iteration)

S.user_cost_func = user_cost_func
S.user_cost_func_arg = itertools.count()
