WIDTH = 8   # Width of inter-FPGA bus
IO_PATTERN = r'^io_'

### Want FPGAs to be named U3 and U4 to match paper example
### Consume U1 and U2 names
S.next_devinstname('U') # U1
S.next_devinstname('U') # U2

# U3
FPGA1_DIN = AddDevice(
    'FPGA1', 'U', ['-25 mm', '0 mm', '0 mm', '0 deg'],
    'fpga/xilinx/xc7vx485t', 'ffg1157', 'ffg1157', 'direct'
)

# U4
FPGA2_DIN = AddDevice(
    'FPGA2', 'U', ['25 mm', '0 mm', '0 mm', '180 deg'],
    'fpga/xilinx/xc7vx485t', 'ffg1157', 'ffg1157', 'direct'
)

S.locked_DINs.add(FPGA1_DIN)
S.locked_DINs.add(FPGA2_DIN)

# AC-coupling caps
for i in range(WIDTH):
    cap_DIN = AddDevice(
        'Cap{}'.format(i), 'C',
        ['0 mm', '{} mm'.format((i - 4) * 2), '0 mm', '0 deg'],
        'passive/c', 'nonpolar', '0402', 'direct'
    )

    S.locked_DINs.add(cap_DIN)


### Add Bundle between FPGA1 and caps
netnames = tuple('A{:}'.format(i) for i in range(WIDTH))
BundleNets('FPGA1_Data', netnames)

AddSimpleEP(
    'FPGA1_Data', 'FPGA1',
    pinfunc_match, [IO_PATTERN], mf_kwargs={
        'ignorecase': True,
    }
)

AddMultigroupEP('FPGA1_Data', (
    (AUTO, 'Cap{}'.format(i), pinfunc_match, ['1'], None, None)
    for i in range(WIDTH)
))



### Add Bundle between FPGA2 and caps
netnames = tuple('B{:}'.format(i) for i in range(WIDTH))
BundleNets('FPGA2_Data', netnames)

AddSimpleEP(
    'FPGA2_Data', 'FPGA2',
    pinfunc_match, [IO_PATTERN], mf_kwargs={
        'ignorecase': True,
    }
)

AddMultigroupEP('FPGA2_Data', (
    (AUTO, 'Cap{}'.format(i), pinfunc_match, ['2'], None, None)
    for i in range(WIDTH)
))

