DDR_PER_FPGA = 2
FPGA_IO_PATTERN = r'^io_'
FPGA_IO_DIFF_PATTERN = r'^io_l\d+p'
FPGA_TXP_PATTERN = r'^mgtxtxp'
FPGA_RXP_PATTERN = r'^mgtxrxp'
DDR_IO_PATTERNS = tuple(
    re.compile(R, flags=re.IGNORECASE) for R in (
        r'^(a|ba|cke|dm|dq|odt|sa)\d+$',
        r'^(cas|cs\d|ras|we|reset|s\d)_n$',
        r'^(scl|sda)$',
    )
)
DDR_IO_DIFF_PATTERNS = tuple(
    re.compile(R, flags=re.IGNORECASE) for R in (
        r'^(ck|dqs)\d+$',
    )
)
PCIE_TXP_PATTERN = r'^hso\d_p$'
PCIE_RXP_PATTERN = r'^hsi\d_p$'
PCIE_WIDTH = 8
ADC_CTL_DIFF_PATTERN = r'^(out\d[ab]_|lclk|adclk|clk)p$'
ADC_CTL_PATTERN = r'^(sync|pdn|reset_n|sclk|cs_n|sdata|sdout)$'
ADC_ANALOG_PATTERN = r'^in\dp$'
DAC_CTL_DIFF_PATTERN = r'^(dacclk|dataclk|parity|d\d+|sync|frame|ostr)p$'
DAC_CTL_PATTERN = r'^(sdo|sdio|sdenb|sclk|txenable|resetb|sleep|alarm)$'
DAC_ANALOG_PATTERN = r'^iout[a-d]p$'
CONN_DIFF_PATTERN = r'^s\dp$'

coords_dontcare = ['0 mm', '0 mm', '0 mm', '0 deg']


FPGA_DIN = AddDevice(
    'FPGA',
    'U', coords_dontcare,
    'fpga/xilinx/xc7vx485t', 'ffg1157', 'ffg1157', 'direct'
)

pcie_DIN = AddDevice(
    'PCIE',
    'P', ['0 mm', '40 mm', '0 mm', '0 deg'],
    'connectors/pcie', 'x8', 'pcie8x', 'direct'
)

ADC_DIN = AddDevice(
    'ADC',
    'U', coords_dontcare,
    'adc/ads5295', 'tqfp80', 'tqfp80', 'direct'
)

for i in range(2):
    DAC_DIN = AddDevice(
        'DAC{}'.format(i),
        'U', coords_dontcare,
        'dac/dac3484', 'nf196', 'pbga196', 'direct'
    )

inconn_DIN = AddDevice(
    'inconn',
    'P', ['-70 mm', '0 mm', '0 mm', '90 deg'],
    'connectors/conn8diff', 's24', 'conn1x24', 'direct'
)

outconn_DIN = AddDevice(
    'outconn',
    'P', ['70 mm', '0 mm', '0 mm', '90 deg'],
    'connectors/conn8diff', 's24', 'conn1x24', 'direct'
)

# Lock connectors in place
S.locked_DINs.add(pcie_DIN)
S.locked_DINs.add(inconn_DIN)
S.locked_DINs.add(outconn_DIN)

for tag, pattern in (
    ('TX', FPGA_TXP_PATTERN),
    ('RX', FPGA_RXP_PATTERN),
):
    AddSimpleEP(
        'Bundle_PCIE{}'.format(tag), 'FPGA',
        pinfunc_match, [pattern], mf_kwargs={
            'ignorecase': True,
        }
    )

    BundleNets(
        'Bundle_PCIE{}'.format(tag),
        tuple(
            'N_PCIE{}<{}>'.format(tag, v)
            for v in range(PCIE_WIDTH)
        )
    )

    BundleSetDifferential(
        'Bundle_PCIE{}'.format(tag),
        '_P',
        '_N'
    )

for tag, pattern in (
    ('TX', PCIE_TXP_PATTERN),
    ('RX', PCIE_RXP_PATTERN),
):
    AddSimpleEP(
        'Bundle_PCIE{}'.format(tag), 'PCIE',
        pinfunc_match, [pattern], mf_kwargs={
            'ignorecase': True,
        }
    )

for i in range(DDR_PER_FPGA):
    DDR_DIN = AddDevice('DDR.{}'.format(i),
              'U', ['0 mm', '0 mm', '0 mm', '0 deg'],
              'ram/ddr3', 'dimm240', 'dimm240_1932000', 'direct')

def match_any_re(devinst, logpinkey, logpin, re_iterable):
    """Return True if any pinfunc matches any compiled RE in re_iterable."""

    for k, pf in logpin.pinfuncs.items():
        for R in re_iterable:
            if R.match(k):
                return True

    return False



# DEBUG: print number of eligible FPGA pins
FPGA_IO_PINS = 0
FPGA_IO_DIFF_PINS = 0
DI = S.devinst_n2o[FPGA_DIN]
for lpk, logpin in DI.logpack.pins.items():
    if pinfunc_match(DI, lpk, logpin, FPGA_IO_PATTERN, ignorecase=True):
        FPGA_IO_PINS += 1
    if pinfunc_match(DI, lpk, logpin, FPGA_IO_DIFF_PATTERN, ignorecase=True):
        FPGA_IO_DIFF_PINS += 1
del DI

print('# of eligible FPGA IO pins:', FPGA_IO_PINS)
print('# of eligible FPGA differential pairs:', FPGA_IO_DIFF_PINS)


# Compute number of nets required to connect to each DDR
DDR_NUM_NETS = 0
DI = S.devinst_n2o[DDR_DIN]
for lpk, logpin in DI.logpack.pins.items():
    if match_any_re(DI, lpk, logpin, DDR_IO_PATTERNS):
        DDR_NUM_NETS += 1
del DI

DDR_NUM_DIFF_NETS = 0
DI = S.devinst_n2o[DDR_DIN]
for lpk, logpin in DI.logpack.pins.items():
    if match_any_re(DI, lpk, logpin, DDR_IO_DIFF_PATTERNS):
        DDR_NUM_DIFF_NETS += 1
del DI

print('DDR: Require', DDR_NUM_NETS, 'single-ended nets')
print('DDR: Require', DDR_NUM_DIFF_NETS, 'differential pairs')


# Compute number of nets required to connect to each ADC
ADC_NUM_NETS = 0
DI = S.devinst_n2o[ADC_DIN]
for lpk, logpin in DI.logpack.pins.items():
    if pinfunc_match(DI, lpk, logpin, ADC_CTL_PATTERN,
                     {'ignorecase': True}):
        ADC_NUM_NETS += 1
del DI

ADC_NUM_DIFF_NETS = 0
DI = S.devinst_n2o[ADC_DIN]
for lpk, logpin in DI.logpack.pins.items():
    if pinfunc_match(DI, lpk, logpin, ADC_CTL_DIFF_PATTERN,
                     {'ignorecase': True}):
        ADC_NUM_DIFF_NETS += 1
del DI

print('ADC: Require', ADC_NUM_NETS, 'single-ended nets')
print('ADC: Require', ADC_NUM_DIFF_NETS, 'differential pairs')


# Compute number of nets required to connect to each DAC
DAC_NUM_NETS = 0
DI = S.devinst_n2o[DAC_DIN]
for lpk, logpin in DI.logpack.pins.items():
    if pinfunc_match(DI, lpk, logpin, DAC_CTL_PATTERN,
                     {'ignorecase': True}):
        DAC_NUM_NETS += 1
del DI

DAC_NUM_DIFF_NETS = 0
DI = S.devinst_n2o[DAC_DIN]
for lpk, logpin in DI.logpack.pins.items():
    if pinfunc_match(DI, lpk, logpin, DAC_CTL_DIFF_PATTERN,
                     {'ignorecase': True}):
        DAC_NUM_DIFF_NETS += 1
del DI

print('DAC: Require', DAC_NUM_NETS, 'single-ended nets')
print('DAC: Require', DAC_NUM_DIFF_NETS, 'differential pairs')


for i in range(DDR_PER_FPGA):
    ### Single-ended nets
    AddSimpleEP(
        'Bundle_DDR.{}'.format(i), 'FPGA',
        pinfunc_match, [FPGA_IO_PATTERN], mf_kwargs={
            'ignorecase': True,
        })

    AddSimpleEP(
        'Bundle_DDR.{}'.format(i), 'DDR.{}'.format(i),
        match_any_re, [DDR_IO_PATTERNS]
    )

    BundleNets(
        'Bundle_DDR.{}'.format(i),
        tuple(
            'N_DDR.{}<{}>'.format(i, v)
            for v in range(DDR_NUM_NETS)
        )
    )


    ### Differential nets
    AddSimpleEP(
        'Bundle_DDR_diff.{}'.format(i), 'FPGA',
        pinfunc_match, [FPGA_IO_DIFF_PATTERN], mf_kwargs={
            'ignorecase': True,
        })

    AddSimpleEP(
        'Bundle_DDR_diff.{}'.format(i), 'DDR.{}'.format(i),
        match_any_re, [DDR_IO_DIFF_PATTERNS]
    )

    BundleNets(
        'Bundle_DDR_diff.{}'.format(i),
        tuple(
            'N_DDR_diff.{}<{}>'.format(i, v)
            for v in range(DDR_NUM_DIFF_NETS)
        )
    )


    BundleSetDifferential(
        'Bundle_DDR_diff.{}'.format(i),
        '_P',
        '_N'
    )



### Single-ended ADC nets
AddSimpleEP(
    'Bundle_ADC', 'FPGA',
    pinfunc_match, [FPGA_IO_PATTERN], mf_kwargs={
        'ignorecase': True,
    })

AddSimpleEP(
    'Bundle_ADC', 'ADC',
    pinfunc_match, [ADC_CTL_PATTERN], mf_kwargs={
        'ignorecase': True,
    })

BundleNets(
    'Bundle_ADC',
    tuple('N_ADC<{}>'.format(i) for i in range(ADC_NUM_NETS))
)


### Differential ADC nets
AddSimpleEP(
    'Bundle_ADC_diff', 'FPGA',
    pinfunc_match, [FPGA_IO_DIFF_PATTERN], mf_kwargs={
        'ignorecase': True,
    })

AddSimpleEP(
    'Bundle_ADC_diff', 'ADC',
    pinfunc_match, [ADC_CTL_DIFF_PATTERN], mf_kwargs={
        'ignorecase': True,
    })

BundleNets(
    'Bundle_ADC_diff',
    tuple('N_ADC_diff<{}>'.format(i) for i in range(ADC_NUM_DIFF_NETS))
)


BundleSetDifferential('Bundle_ADC_diff', '_P', '_N')



### Single-ended DAC nets
AddSimpleEP(
    'Bundle_DAC', 'FPGA',
    pinfunc_match, [FPGA_IO_PATTERN], mf_kwargs={
        'ignorecase': True,
    })

AddMultigroupEP(
    'Bundle_DAC', (
        (
            DAC_NUM_NETS, 'DAC0', pinfunc_match,
            [DAC_CTL_PATTERN], {'ignorecase': True}, None,
        ),
        (
            DAC_NUM_NETS, 'DAC1', pinfunc_match,
            [DAC_CTL_PATTERN], {'ignorecase': True}, None,
        ),
    )
)

BundleNets(
    'Bundle_DAC',
    tuple('N_DAC<{}>'.format(i) for i in range(2*DAC_NUM_NETS))
)



### Differential DAC nets
AddSimpleEP(
    'Bundle_DAC_diff', 'FPGA',
    pinfunc_match, [FPGA_IO_DIFF_PATTERN], mf_kwargs={
        'ignorecase': True,
    })

AddMultigroupEP(
    'Bundle_DAC_diff', (
        (
            DAC_NUM_DIFF_NETS, 'DAC0', pinfunc_match,
            [DAC_CTL_DIFF_PATTERN], {'ignorecase': True}, None,
        ),
        (
            DAC_NUM_DIFF_NETS, 'DAC1', pinfunc_match,
            [DAC_CTL_DIFF_PATTERN], {'ignorecase': True}, None,
        ),
    )
)

BundleNets(
    'Bundle_DAC_diff',
    tuple('N_DAC_diff<{}>'.format(i) for i in range(2*DAC_NUM_DIFF_NETS))
)


BundleSetDifferential('Bundle_DAC_diff', '_P', '_N')



### Analog inputs
AddSimpleEP(
    'Bundle_inputs', 'inconn',
    pinfunc_match, [CONN_DIFF_PATTERN], mf_kwargs={
        'ignorecase': True,
    })

AddSimpleEP(
    'Bundle_inputs', 'ADC',
    pinfunc_match, [ADC_ANALOG_PATTERN], mf_kwargs={
        'ignorecase': True,
    })

BundleNets(
    'Bundle_inputs',
    tuple('N_in<{}>'.format(i) for i in range(8))
)

BundleSetDifferential('Bundle_inputs', '_P', '_N')
BundleParams('Bundle_inputs', {'weight': 1})



### Analog outputs
AddSimpleEP(
    'Bundle_outputs', 'outconn',
    pinfunc_match, [CONN_DIFF_PATTERN], mf_kwargs={
        'ignorecase': True,
    })

AddMultigroupEP(
    'Bundle_outputs', (
        (
            4, 'DAC0', pinfunc_match,
            [DAC_ANALOG_PATTERN], {'ignorecase': True}, None,
        ),
        (
            4, 'DAC1', pinfunc_match,
            [DAC_ANALOG_PATTERN], {'ignorecase': True}, None,
        ),
    )
)

BundleNets(
    'Bundle_outputs',
    tuple('N_out<{}>'.format(i) for i in range(8))
)

BundleSetDifferential('Bundle_outputs', '_P', '_N')
BundleParams('Bundle_outputs', {'weight': 1})
