#!/usr/bin/env python3

"""Device library: ADC: AD7490"""

import device



pins_to_create = (
    (device.POWER, device.INPUT, True, (
        'VDD',
        'AGND',
        'VDRIVE',
        'PAD',
    )),

    (device.DIGITAL, device.INPUT, True, (
        'SCLK',
        'DIN',
        'CS_n',
    )),

    (device.DIGITAL, device.OUTPUT, True, (
        'DOUT',
    )),

    (device.ANALOG, device.INPUT, True, (
        'REFIN',
    )),

    (device.ANALOG, device.INPUT, True, tuple(
        'VIN{}'.format(i) for i in range(16)
    )),
)


pack2pinnum2name = {
    'tssop28': {
        20: 'CS_n',
        23: 'REFIN',
        22: 'VDD',
        14: 'AGND',
        21: 'AGND',
        24: 'AGND',
        13: 'VIN0',
        12: 'VIN1',
        11: 'VIN2',
        10: 'VIN3',
        9: 'VIN4',
        8: 'VIN5',
        7: 'VIN6',
        6: 'VIN7',
        5: 'VIN8',
        3: 'VIN9',
        2: 'VIN10',
        1: 'VIN11',
        28: 'VIN12',
        27: 'VIN13',
        26: 'VIN14',
        25: 'VIN15',
        19: 'DIN',
        15: 'DOUT',
        16: 'SCLK',
        17: 'VDRIVE',
        4: None,
        18: None,
    },

    'lfcsp32': {
        18: 'CS_n',
        21: 'REFIN',
        20: 'VDD',
        12: 'AGND',
        19: 'AGND',
        22: 'AGND',
        11: 'VIN0',
        10: 'VIN1',
        9: 'VIN2',
        7: 'VIN3',
        6: 'VIN4',
        5: 'VIN5',
        4: 'VIN6',
        3: 'VIN7',
        2: 'VIN8',
        31: 'VIN9',
        30: 'VIN10',
        29: 'VIN11',
        28: 'VIN12',
        27: 'VIN13',
        26: 'VIN14',
        24: 'VIN15',
        17: 'DIN',
        13: 'DOUT',
        14: 'SCLK',
        15: 'VDRIVE',
        33: 'PAD',
        1: None,
        8: None,
        16: None,
        23: None,
        25: None,
        32: None,
    },
}


sigconfig = device.SINGLE

packages = dict()

for packname, pinnum2name in pack2pinnum2name.items():
    Xpins = dict()

    for sigtype, sigdir, required, names in pins_to_create:
        for n in names:
            Xpins[n] = device.LogicalPin(pinfuncs={
                n   :   device.PinFunc(sigtype, sigdir, sigconfig,
                                       required=required)
            })


    pindict = dict()
    for pinnum, pinname in pinnum2name.items():
        if pinname:
            pindict[pinnum] = Xpins[pinname]
        else:
            pindict[pinnum] = device.LogicalPin()




    pack = device.LogicalPackage(pins=pindict)

    packages[packname] = pack




dev = device.Device(packages=packages)
