#!/usr/bin/env python3

"""Device library: ADC: ADS5517 (LVDS mode)"""

import device



pins_to_create = (
    (device.POWER, device.INPUT, True, (
        'AVDD',
        'AGND',
        'DRVDD',
        'DRGND',
        'PAD',
    )),

    (device.DIGITAL, device.INPUT, True, (
        'CLKP',
        'CLKM',
        'RESET',
        'SCLK',
        'SDATA',
        'SEN',
        'OE',
        'DFS',
        'MODE',
    )),

    (device.DIGITAL, device.OUTPUT, False, (
        'CLKOUTP',
        'CLKOUTM',
        'LOW_D0_P',
        'LOW_D0_M',
        'OVR',
    )),

    (device.DIGITAL, device.OUTPUT, False, tuple(
        'D{:}_D{:}_P'.format(i, i+1)
        for i in range(1, 10, 2)
    )),

    (device.DIGITAL, device.OUTPUT, False, tuple(
        'D{:}_D{:}_M'.format(i, i+1)
        for i in range(1, 10, 2)
    )),

    (device.ANALOG, device.INPUT, True, (
        'INP',
        'INM',
        'IREF',
    )),

    (device.ANALOG, device.BIDIR, False, (
        'VCM',
    )),
)


pinnum2name = {
    0   :   'PAD',
    1   :   'DRGND',
    2   :   'DRVDD',
    3   :   'OVR',
    4   :   'CLKOUTM',
    5   :   'CLKOUTP',
    6   :   'DFS',
    7   :   'OE',
    8   :   'AVDD',
    9   :   'AGND',
    10  :   'CLKP',
    11  :   'CLKM',
    12  :   'AGND',
    13  :   'VCM',
    14  :   'AGND',
    15  :   'INP',
    16  :   'INM',
    17  :   'AGND',
    18  :   'AVDD',
    19  :   'AGND',
    20  :   'AVDD',
    21  :   'IREF',
    22  :   'AVDD',
    23  :   'MODE',
    24  :   'AVDD',
    25  :   'AGND',
    26  :   'AVDD',
    27  :   'SEN',
    28  :   'SDATA',
    29  :   'SCLK',
    30  :   'RESET',
    35  :   'DRVDD',
    36  :   'DRGND',
    37  :   'LOW_D0_M',
    38  :   'LOW_D0_P',
    39  :   'D1_D2_M',
    40  :   'D1_D2_P',
    41  :   'D3_D4_M',
    42  :   'D3_D4_P',
    43  :   'D5_D6_M',
    44  :   'D5_D6_P',
    45  :   'D7_D8_M',
    46  :   'D7_D8_P',
    47  :   'D9_D10_M',
    48  :   'D9_D10_P',
}


sigconfig = device.SINGLE


Xpins = dict()

for sigtype, sigdir, required, names in pins_to_create:
    for n in names:
        Xpins[n] = device.LogicalPin(pinfuncs = {
            n   :   device.PinFunc(sigtype, sigdir, sigconfig,
                                   required = required)
        })


pindict = dict()
for pinnum, pinname in pinnum2name.items():
    pindict[pinnum] = Xpins[pinname]




pack48 = device.LogicalPackage(pins = pindict)




dev = device.Device(packages = {
    'QFN-48'    :   pack48,
})
