#!/usr/bin/env python3

"""Device library: DAC: AD5360"""

import device



pins_to_create = (
    (device.POWER, device.INPUT, True, (
        'VDD',
        'VSS',
        'DGND',
        'DVCC',
        'AGND',
        'PAD',
    )),

    (device.DIGITAL, device.INPUT, True, (
        'LDAC_n',
        'CLR_n',
        'RESET_n',
        'BIN_n/2SCOMP',
        'SYNC_n',
        'SCLK',
        'SDI',
    )),

    (device.DIGITAL, device.BIDIR, True, (
        'GPIO',
        'BUSY_n',
    )),

    (device.DIGITAL, device.OUTPUT, True, (
        'PEC_n',
        'SDO',
    )),

    (device.ANALOG, device.INPUT, True, (
        'MON_IN0',
        'MON_IN1',
        'VREF0',
        'VREF1',
        'SIGGND0',
        'SIGGND1',
    )),

    (device.ANALOG, device.OUTPUT, True, (
        'MON_OUT',
        'TEMP_OUT'
    )),

    (device.ANALOG, device.OUTPUT, True, tuple(
        'VOUT{}'.format(i) for i in range(16)
    )),
)


pack2pinnum2name = {
    'lqfp52': {
        1: 'LDAC_n',
        2: 'CLR_n',
        3: 'RESET_n',
        4: 'BIN_n/2SCOMP',
        5: 'BUSY_n',
        6: 'GPIO',
        7: 'MON_OUT',
        8: 'MON_IN0',
        32: 'MON_IN1',
        11: 'VDD',
        28: 'VDD',
        12: 'VSS',
        29: 'VSS',
        13: 'VREF1',
        19: 'SIGGND1',
        31: 'VREF0',
        33: 'TEMP_OUT',
        34: 'VOUT0',
        35: 'VOUT1',
        36: 'VOUT2',
        37: 'VOUT3',
        39: 'VOUT4',
        40: 'VOUT5',
        41: 'VOUT6',
        42: 'VOUT7',
        15: 'VOUT8',
        16: 'VOUT9',
        17: 'VOUT10',
        18: 'VOUT11',
        20: 'VOUT12',
        21: 'VOUT13',
        22: 'VOUT14',
        23: 'VOUT15',
        38: 'SIGGND0',
        43: 'DGND',
        51: 'DGND',
        44: 'DVCC',
        50: 'DVCC',
        45: 'SYNC_n',
        46: 'SCLK',
        47: 'SDI',
        48: 'PEC_n',
        49: 'SDO',
        52: 'AGND',
        9: None,
        10: None,
        14: None,
        24: None,
        25: None,
        26: None,
        27: None,
        30: None,
    },

    'lfcsp56': {
        55: 'LDAC_n',
        56: 'CLR_n',
        1: 'RESET_n',
        2: 'BIN_n/2SCOMP',
        3: 'BUSY_n',
        4: 'GPIO',
        5: 'MON_OUT',
        6: 'MON_IN0',
        34: 'MON_IN1',
        12: 'VDD',
        29: 'VDD',
        13: 'VSS',
        30: 'VSS',
        14: 'VREF1',
        21: 'SIGGND1',
        33: 'VREF0',
        35: 'TEMP_OUT',
        36: 'VOUT0',
        37: 'VOUT1',
        38: 'VOUT2',
        39: 'VOUT3',
        41: 'VOUT4',
        42: 'VOUT5',
        43: 'VOUT6',
        44: 'VOUT7',
        17: 'VOUT8',
        18: 'VOUT9',
        19: 'VOUT10',
        20: 'VOUT11',
        22: 'VOUT12',
        23: 'VOUT13',
        24: 'VOUT14',
        25: 'VOUT15',
        40: 'SIGGND0',
        45: 'DGND',
        53: 'DGND',
        46: 'DVCC',
        52: 'DVCC',
        47: 'SYNC_n',
        48: 'SCLK',
        49: 'SDI',
        50: 'PEC_n',
        51: 'SDO',
        54: 'AGND',
        57: 'PAD',
        7: None,
        8: None,
        9: None,
        10: None,
        11: None,
        15: None,
        16: None,
        26: None,
        27: None,
        28: None,
        31: None,
        32: None,
    },
}


sigconfig = device.SINGLE


packages = dict()

for packname, pinnum2name in pack2pinnum2name.items():
    Xpins = dict()

    for sigtype, sigdir, required, names in pins_to_create:
        for n in names:
            Xpins[n] = device.LogicalPin(pinfuncs={
                n   :   device.PinFunc(sigtype, sigdir, sigconfig,
                                       required=required)
            })


    pindict = dict()
    for pinnum, pinname in pinnum2name.items():
        if pinname:
            pindict[pinnum] = Xpins[pinname]
        else:
            pindict[pinnum] = device.LogicalPin()




    pack = device.LogicalPackage(pins=pindict)

    packages[packname] = pack




dev = device.Device(packages=packages)
