#!/usr/bin/env python3

"""Device library: microcontrollers: ATtiny25, 45, 85"""

import device


Pin_pb5 = device.LogicalPin(pinfuncs = {
    'PB5'       :   device.PinFunc(device.DIGITAL, device.BIDIR,
                                  device.SINGLE),
    'PCINT5'    :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
    'RESET_n'   :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
    'ADC0'      :   device.PinFunc(device.ANALOG, device.INPUT,
                                  device.SINGLE),
    'dW'        :   device.PinFunc(device.DIGITAL, device.BIDIR,
                                  device.SINGLE),
})

Pin_pb3 = device.LogicalPin(pinfuncs = {
    'PB3'       :   device.PinFunc(device.DIGITAL, device.BIDIR,
                                  device.SINGLE),
    'PCINT3'    :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
    'XTAL1'     :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
    'CLKI'      :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
    'OC1B_n'    :   device.PinFunc(device.DIGITAL, device.OUTPUT,
                                  device.SINGLE),
    'ADC3'      :   device.PinFunc(device.ANALOG, device.INPUT,
                                  device.SINGLE),
})

Pin_pb4 = device.LogicalPin(pinfuncs = {
    'PB4'       :   device.PinFunc(device.DIGITAL, device.BIDIR,
                                  device.SINGLE),
    'PCINT4'    :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
    'XTAL2'     :   device.PinFunc(device.DIGITAL, device.OUTPUT,
                                  device.SINGLE),
    'CLKO'      :   device.PinFunc(device.DIGITAL, device.OUTPUT,
                                  device.SINGLE),
    'OC1B'      :   device.PinFunc(device.DIGITAL, device.OUTPUT,
                                  device.SINGLE),
    'ADC2'      :   device.PinFunc(device.ANALOG, device.INPUT,
                                  device.SINGLE),
})

Pin_pb2 = device.LogicalPin(pinfuncs = {
    'PB2'       :   device.PinFunc(device.DIGITAL, device.BIDIR,
                                  device.SINGLE),
    'SCK'       :   device.PinFunc(device.DIGITAL, device.OUTPUT,
                                  device.SINGLE),
    'USCK'      :   device.PinFunc(device.DIGITAL, device.OUTPUT,
                                  device.SINGLE),
    'SCL'       :   device.PinFunc(device.DIGITAL, device.BIDIR,
                                  device.SINGLE),
    'ADC1'      :   device.PinFunc(device.ANALOG, device.INPUT,
                                  device.SINGLE),
    'T0'        :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
    'INT0'      :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
    'PCINT2'    :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
})

Pin_pb1 = device.LogicalPin(pinfuncs = {
    'PB1'       :   device.PinFunc(device.DIGITAL, device.BIDIR,
                                  device.SINGLE),
    'MISO'      :   device.PinFunc(device.DIGITAL, device.OUTPUT,
                                  device.SINGLE),
    'DO'        :   device.PinFunc(device.DIGITAL, device.OUTPUT,
                                  device.SINGLE),
    'AIN1'      :   device.PinFunc(device.ANALOG, device.INPUT,
                                  device.SINGLE),
    'OC0B'      :   device.PinFunc(device.DIGITAL, device.OUTPUT,
                                  device.SINGLE),
    'OC1A'      :   device.PinFunc(device.DIGITAL, device.OUTPUT,
                                  device.SINGLE),
    'PCINT1'    :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
})

Pin_pb0 = device.LogicalPin(pinfuncs = {
    'PB0'       :   device.PinFunc(device.DIGITAL, device.BIDIR,
                                  device.SINGLE),
    'MOSI'      :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
    'DI'        :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
    'SDA'       :   device.PinFunc(device.DIGITAL, device.BIDIR,
                                  device.SINGLE),
    'AIN0'      :   device.PinFunc(device.ANALOG, device.INPUT,
                                  device.SINGLE),
    'OC0A'      :   device.PinFunc(device.DIGITAL, device.OUTPUT,
                                  device.SINGLE),
    'OC1A_n'    :   device.PinFunc(device.DIGITAL, device.OUTPUT,
                                  device.SINGLE),
    'AREF'      :   device.PinFunc(device.ANALOG, device.INPUT,
                                  device.SINGLE),
    'PCINT0'    :   device.PinFunc(device.DIGITAL, device.INPUT,
                                  device.SINGLE),
})


Pin_GND = device.LogicalPin(pinfuncs = {
    'GND'       :   device.PinFunc(device.POWER, device.INPUT,
                                   device.SINGLE,
                                   required=True),
})


Pin_VCC = device.LogicalPin(pinfuncs = {
    'VCC'       :   device.PinFunc(device.POWER, device.INPUT,
                                   device.SINGLE, required=True),
})


pack8 = device.LogicalPackage(pins = {
    1   :   Pin_pb5,
    2   :   Pin_pb3,
    3   :   Pin_pb4,
    4   :   Pin_GND,
    5   :   Pin_pb0,
    6   :   Pin_pb1,
    7   :   Pin_pb2,
    8   :   Pin_VCC,
})


pack20 = device.LogicalPackage(pins = {
    1   :   Pin_pb5,
    2   :   Pin_pb3,
    5   :   Pin_pb4,
    8   :   Pin_GND,
    11  :   Pin_pb0,
    12  :   Pin_pb1,
    14  :   Pin_pb2,
    15  :   Pin_VCC,
    'PAD':  Pin_GND,
})



dev = device.Device(packages = {
    'DIP-8'     :   pack8,
    'SO-8'      :   pack8,
    'TSSOP-8'   :   pack8,
    'QFN-20'    :   pack20,
    'MLF-20'    :   pack20,
})
