#!/usr/bin/env python3

"""PhysicalPackage: surface mount 0402"""

import fractions
import wallpy.units

import device


part_dimensions = (
    wallpy.units.Length(fractions.Fraction(13, 10), 'mm'),
    wallpy.units.Length(fractions.Fraction(6, 10), 'mm'),
    wallpy.units.Length(fractions.Fraction(35, 100), 'mm'),
)


pad_dimensions = (
    wallpy.units.Length(fractions.Fraction(4, 10), 'mm'),
    wallpy.units.Length(fractions.Fraction(6, 10), 'mm'),
    wallpy.units.Length(0, 'mm'),
)

pad_spacing = wallpy.units.Length(fractions.Fraction(1, 2), 'mm')

x_displacement = (pad_dimensions[0] + pad_spacing) / 2




phypack = device.PhysicalPackage(
    part_dimensions,

    pins = {
        1 : device.PhysicalPin(
            (
                -x_displacement,
                wallpy.units.Length(0, 'mm'),
                wallpy.units.Length(0, 'mm'),
            ),
            dimensions = pad_dimensions
        ),

        2 : device.PhysicalPin(
            (
                x_displacement,
                wallpy.units.Length(0, 'mm'),
                wallpy.units.Length(0, 'mm'),
            ),
            dimensions = pad_dimensions
        ),

    }
)
