#!/usr/bin/env python3

"""PhysicalPackage: 1 row by 24 pin connector"""

import fractions
import wallpy.units

import device


NUM_PINS = 24
PIN_SPACING = wallpy.units.Length(fractions.Fraction(1, 2), 'mm')

part_dimensions = (
    (NUM_PINS + 1) * PIN_SPACING,
    2 * PIN_SPACING,
    wallpy.units.Length(fractions.Fraction(2, 1), 'mm'),
)

pin_dimensions = (
    PIN_SPACING / 2,
    PIN_SPACING / 2,
    wallpy.units.Length(0, 'mm'),
)

center_index = (NUM_PINS - 1) / 2
y_offset = wallpy.units.Length(0, 'mm')

def new_pin(index):
    """Generate new PhysicalPin given 0-based pin number."""

    x = (index - center_index) * PIN_SPACING
    y = y_offset

    return device.PhysicalPin(
        (x, y, wallpy.units.Length(0, 'mm')),
        dimensions=pin_dimensions,
        attr={'shape': 'circ'}
    )


pins = dict()
for i in range(NUM_PINS):
    pins[i+1] = new_pin(i)

phypack = device.PhysicalPackage(part_dimensions, pins=pins)
