#!/usr/bin/env python3

"""PhysicalPackage: through-hole DDR2 DIMM socket, part 87705"""

import fractions
import wallpy.units

import device


part_dimensions = (
    wallpy.units.Length(fractions.Fraction(148, 1), 'mm'),
    wallpy.units.Length(fractions.Fraction(735, 100), 'mm'),
    wallpy.units.Length(fractions.Fraction(235, 10), 'mm'),
)


pin_dimensions = (
    wallpy.units.Length(fractions.Fraction(7, 10), 'mm'),
    wallpy.units.Length(fractions.Fraction(7, 10), 'mm'),
    wallpy.units.Length(0, 'mm'),
)

pin_spacing = wallpy.units.Length(fractions.Fraction(1, 1), 'mm')

# Offset from center of innermost west, east pin
x_offset_west = wallpy.units.Length(fractions.Fraction(-615, 10), 'mm')
x_offset_east = wallpy.units.Length(fractions.Fraction(65, 10), 'mm')

# Offsets of inner and outer rows of pins
y_offset_inner = wallpy.units.Length(fractions.Fraction(95, 100), 'mm')
y_offset_outer = wallpy.units.Length(fractions.Fraction(285, 100), 'mm')

NUM_PINS = 240  # Total number of pins
FIRST_PIN_AFTER_SLOT = 64   # Zero-based index of first pin after card slot


def new_pin(pinnum):
    """Generate new PhysicalPin given 0-based pin number."""

    x_index = pinnum % (NUM_PINS / 2)
    side = pinnum // (NUM_PINS / 2)

    if x_index < FIRST_PIN_AFTER_SLOT:
        # West side of slot
        x = x_offset_west + (x_index - 0) * pin_spacing
    else:
        x = x_offset_east + (x_index - FIRST_PIN_AFTER_SLOT) * pin_spacing

    if (x_index % 2) == 0:
        y = y_offset_inner
    else:
        y = y_offset_outer

    if side > 0:
        # Shift north above center
        y -= (y_offset_inner + y_offset_outer)

    return device.PhysicalPin(
        (x, y, wallpy.units.Length(0, 'mm')),
        dimensions=pin_dimensions,
        attr={'shape': 'circ'}
    )


phypack = device.PhysicalPackage(
    part_dimensions,
    pins={i+1: new_pin(i) for i in range(NUM_PINS)}
)
