#!/usr/bin/env python3

"""PhysicalPackage: surface mount HC-49/US crystal"""

import fractions
import wallpy.units

import device


part_dimensions = (
    wallpy.units.Length(fractions.Fraction(151, 10), 'mm'),
    wallpy.units.Length(fractions.Fraction(48, 10), 'mm'),
    wallpy.units.Length(fractions.Fraction(42, 10), 'mm'),
)


pad_dimensions = (
    wallpy.units.Length(fractions.Fraction(56, 10), 'mm'),
    wallpy.units.Length(fractions.Fraction(21, 10), 'mm'),
    wallpy.units.Length(0, 'mm'),
)

x_displacement = wallpy.units.Length(fractions.Fraction(95, 20), 'mm')




phypack = device.PhysicalPackage(
    part_dimensions,

    pins = {
        1 : device.PhysicalPin(
            (
                -x_displacement,
                wallpy.units.Length(0, 'mm'),
                wallpy.units.Length(0, 'mm'),
            ),
            dimensions = pad_dimensions
        ),

        2 : device.PhysicalPin(
            (
                x_displacement,
                wallpy.units.Length(0, 'mm'),
                wallpy.units.Length(0, 'mm'),
            ),
            dimensions = pad_dimensions
        ),

    }
)
