#!/usr/bin/env python3

"""PhysicalPackage: through-hole PCIe x8 socket, part PCIE-098-02-X-D-TH"""

# XXX: Approximation of actual socket! Real socket has two holes per pin!

import fractions
import wallpy.units

import device


PINS_PER_ROW = 49
FIRST_PIN_AFTER_SLOT = 11   # Zero-based index of first pin after card slot

part_dimensions = (
    wallpy.units.Length(fractions.Fraction(56, 1), 'mm'),
    wallpy.units.Length(fractions.Fraction(86, 10), 'mm'),
    wallpy.units.Length(fractions.Fraction(10, 1), 'mm'),
)


pin_dimensions = (
    wallpy.units.Length(fractions.Fraction(35, 100), 'mm'),
    wallpy.units.Length(fractions.Fraction(35, 100), 'mm'),
    wallpy.units.Length(0, 'mm'),
)

# Pins count upward while moving west
pin_spacing = wallpy.units.Length(fractions.Fraction(-1, 1), 'mm')

slot_extra_width = wallpy.units.Length(fractions.Fraction(-2, 1), 'mm')

# Offset from center to eastmost pin
x_offset_c2e = wallpy.units.Length(fractions.Fraction(PINS_PER_ROW, 2), 'mm')

# Offsets of A and B rows of pins
y_offsets = {
    'A': wallpy.units.Length(fractions.Fraction(223, 100), 'mm'),
    'B': wallpy.units.Length(fractions.Fraction(-223, 100), 'mm'),
}


def new_pin(row, index):
    """Generate new PhysicalPin given row (A or B) and 0-based pin number."""

    x = x_offset_c2e + (index * pin_spacing)
    if index >= FIRST_PIN_AFTER_SLOT:
        x += slot_extra_width

    y = y_offsets[row]

    return device.PhysicalPin(
        (x, y, wallpy.units.Length(0, 'mm')),
        dimensions=pin_dimensions,
        attr={'shape': 'circ'}
    )


pins = dict()
for row in ('A', 'B'):
    for i in range(PINS_PER_ROW):
        pins['{}{}'.format(row, i+1)] = new_pin(row, i)

phypack = device.PhysicalPackage(part_dimensions, pins=pins)
