#!/usr/bin/env python3

import sys
import re
import itertools
import wallpy.units

import schem

FPGA_DEV_FN = 'fpga/xilinx/xc7vx485t'
NETS_PER_PAIR = 32
NUM_FPGAS = 6

IO_PIN_SET = frozenset({
    'AA34', 'Y24', 'V30', 'W26',
    'Y32', 'Y34', 'AB28', 'AA25',
    'W24', 'W29', 'U30',
})

devinstances = list()

fpga_DINs = tuple('U{:01}'.format(i+1) for i in range(NUM_FPGAS))

for i, DIN in enumerate(fpga_DINs):
    x = wallpy.units.Length((i % 2) * 50, 'mm')
    y = wallpy.units.Length((i // 2) * 50, 'mm')
    z = wallpy.units.Length(0, 'mm')
    angle = wallpy.units.Angle(0, 'deg')
    coords = (x, y, z, angle)

    devinstances.append((
        DIN,
        coords,
        FPGA_DEV_FN,
        'ffg1157',
        'ffg1157',
        'direct',
    ))

S = schem.Schematic(devinstances=devinstances)


re_io = re.compile(r'^io', flags=re.IGNORECASE)
re_t = re.compile(r'^t[0-9]$', flags=re.IGNORECASE)

def match_io_only(devinst, logpinkey, logpin, bank):
    """Return True only for pins having only IO* and T? pinfuncs.

    Optionally restrict to pins with given bank.

    Extra arguments:
    bank -- return pins in only this bank (specify None as don't-care)

    """

    have_io = False

    for pfname, pf in logpin.pinfuncs.items():
        if re_io.match(pfname):
            if bank is None:
                have_io = True

            elif pf.params['bank'] == bank:
                    print('found pin', logpinkey)
                    have_io = True

        elif re_t.match(pfname):
            pass

        else:
            return False

    return have_io


def match_pin_in_set(devinst, logpinkey, logpin, acceptset):
    return logpinkey in acceptset


for pair_DINs in itertools.combinations(fpga_DINs, 2):
    S.add_bundle(
        ''.join(('B_', ''.join(pair_DINs))),
        [
            ''.join(('n_', ''.join(pair_DINs), '_', str(i)))
            for i in range(NETS_PER_PAIR)
        ],
        [
            [
                (
                    schem.AUTO,
                    (
                        DIN,
                        match_io_only,
                        [None],
                        {},
                    ),
                    {
                    }
                ),
            ]

            for DIN in pair_DINs
        ]
    )


print('solve_ilp:', S.solve_ilp())

S.export(sys.stdout)

svg_prefs = {
    'zoom': 5,
}

with open('/tmp/huge.svg', 'w') as F:
    S.export_svg(F, prefs=svg_prefs)

S.debug_bundles()
