#!/usr/bin/env python3

import unittest
import sys
import os.path

import devimport
import device




class Test_importtext(unittest.TestCase):

    def setUp(self):
        pass


    def test_LED(self):
        text = """
1	ANODE=ai,<REQUIRED>
2	CATHODE=ai,<REQUIRED>
""".strip()

        expect = device.LogicalPackage(pins = {
            1 : device.LogicalPin(pinfuncs = {
                'ANODE' : device.PinFunc(
                    device.ANALOG, device.INPUT, device.SINGLE,
                    required=True,
                ),
            }),
            2 : device.LogicalPin(pinfuncs = {
                'CATHODE' : device.PinFunc(
                    device.ANALOG, device.INPUT, device.SINGLE,
                    required=True,
                ),
            }),
        })

        actual = devimport.lines2package(text.split('\n'))

        self.assertEqual(expect, actual, 'imported package differs')


    def test_Diff(self):
        text = """
1	Ap=ai,<REQUIRED>,<DIFF_P>,<DIFF_PAIR>=An
2	An=ai,<REQUIRED>,<DIFF_N>,<DIFF_PAIR>=Ap
3	Bp=ao,<REQUIRED>,<DIFF_P>,<DIFF_PAIR>=Bn
4	Bn=ao,<REQUIRED>,<DIFF_N>,<DIFF_PAIR>=Bp
""".strip()

        expect = device.LogicalPackage(pins={
            1: device.LogicalPin(pinfuncs={
                'Ap': device.PinFunc(
                    device.ANALOG, device.INPUT, device.DIFF_P,
                    required=True,
                    params={
                        'diff_pair': 'An',
                    },
                ),
            }),
            2: device.LogicalPin(pinfuncs={
                'An': device.PinFunc(
                    device.ANALOG, device.INPUT, device.DIFF_N,
                    required=True,
                    params={
                        'diff_pair': 'Ap',
                    },
                ),
            }),
            3: device.LogicalPin(pinfuncs={
                'Bp': device.PinFunc(
                    device.ANALOG, device.OUTPUT, device.DIFF_P,
                    required=True,
                    params={
                        'diff_pair': 'Bn',
                    },
                ),
            }),
            4: device.LogicalPin(pinfuncs={
                'Bn': device.PinFunc(
                    device.ANALOG, device.OUTPUT, device.DIFF_N,
                    required=True,
                    params={
                        'diff_pair': 'Bp',
                    },
                ),
            }),
        })

        actual = devimport.lines2package(text.split('\n'))

        self.assertEqual(expect, actual, 'imported package differs')



__TL = unittest.TestLoader()

alltests = unittest.TestSuite((
    __TL.loadTestsFromTestCase(Test_importtext),
))


if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(alltests)

