#!/usr/bin/env python3

import unittest
import sys
import os.path
import fractions

import wallpy.random
import wallpy.units
import wallpy.cleanobject

import device
import devmanager




class Test_devmanager(unittest.TestCase):

    def setUp(self):
        self.RD = wallpy.random.RandomDistinct(1000, 10000)


    def test_log2phy_direct(self):
        test_values = [
            next(self.RD),
            next(self.RD),
            next(self.RD),
            next(self.RD),
            None,
            (next(self.RD), next(self.RD)),
            (next(self.RD), next(self.RD)),
            object(),
            object(),
        ]

        CM = devmanager.load_log2phy('direct')

        for v in test_values:
            expect = (v, )
            self.assertTupleEqual(expect, CM[v], 'direct mismatch')


    def test_log2phy_ak(self):
        expect = {
            'fish'  : tuple(),
            'cow'   : tuple(),
        }

        for i in 'a A anode aNOdE ANODE'.split():
            expect[i] = (1, )

        for i in 'k K cathode cAtHode CATHODE'.split():
            expect[i] = (2, )

        CM = devmanager.load_log2phy('ak')


        actual = {k : CM[k] for k in expect}

        self.assertDictEqual(expect, actual, 'ak mismatch')



    def test_phypack_0402(self):

        expect = device.PhysicalPackage(
            # dimensions = 
            (
                wallpy.units.Length(fractions.Fraction(1, 1), 'mm'),
                wallpy.units.Length(fractions.Fraction(1, 2), 'mm'),
                wallpy.units.Length(fractions.Fraction(35, 100), 'mm'),
            ),
            pins = 
            wallpy.cleanobject.CleanDict({
                1 : device.PhysicalPin(
                    # coords = 
                    (
                        wallpy.units.Length(fractions.Fraction(-45, 100), 'mm'),
                        wallpy.units.Length(fractions.Fraction(0, 1), 'mm'),
                        wallpy.units.Length(fractions.Fraction(0, 1), 'mm'),
                    ),
                    dimensions = 
                    (
                        wallpy.units.Length(fractions.Fraction(4, 10), 'mm'),
                        wallpy.units.Length(fractions.Fraction(6, 10), 'mm'),
                        wallpy.units.Length(fractions.Fraction(0, 1), 'mm'),
                    )
                ),


                2 : device.PhysicalPin(
                    # coords = 
                    (
                        wallpy.units.Length(fractions.Fraction(45, 100), 'mm'),
                        wallpy.units.Length(fractions.Fraction(0, 1), 'mm'),
                        wallpy.units.Length(fractions.Fraction(0, 1), 'mm'),
                    ),
                    dimensions = 
                    (
                        wallpy.units.Length(fractions.Fraction(4, 10), 'mm'),
                        wallpy.units.Length(fractions.Fraction(6, 10), 'mm'),
                        wallpy.units.Length(fractions.Fraction(0, 1), 'mm'),
                    )
                ),
            })
        )



        actual = devmanager.load_phypack('0402')




__TL = unittest.TestLoader()

alltests = unittest.TestSuite((
    __TL.loadTestsFromTestCase(Test_devmanager),
))


if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(alltests)

