#!/usr/bin/env python3

import unittest
import sys
import os.path

import wallpy.viz
import wallpy.version

import devmanager




devpath = 'fpga/xilinx'



class Test_xc6slx9(unittest.TestCase):
    partnum = 'xc6slx9'

    def setUp(self):
        self.D = devmanager.load_device(os.path.join(devpath, self.partnum))


    def test_valid(self):
        self.assertEqual(self.D.valid_p(errcallback=print), True,
                         'supposed to be valid')


    def test_required_pins(self):
        pack = self.D.packages['tqg144']

        expect = frozenset((
            3, 4, 13, 18, 19, 20, 25, 28, 31, 36, 37, 39, 42, 49,
            52, 53, 54, 60, 63, 68, 69, 70, 73, 76, 77, 86, 89, 90, 91,
            96, 103, 106, 107, 108, 109, 110, 113, 122, 125, 128, 129,
            130, 135, 136, 144,
        ))

        actual = frozenset(filter(lambda k : pack.pins[k].required_p(), pack.pins))

        self.assertSetEqual(expect, actual, 'mismatch on required pins')



class Test_xc3s200a(unittest.TestCase):
    partnum = 'xc3s200a'
    packnum = 'vq100'

    def setUp(self):
        self.D = devmanager.load_device(os.path.join(devpath, self.partnum))


    def test_viz(self):
        pack = self.D.packages[self.packnum]


        def pindesc(pin):
            def pinfuncs_desc(pin):
                for k, pinfunc in pin.pinfuncs.items():
                    if pinfunc.required:
                        yield k + ' ***'
                    else:
                        yield k

            return ', '.join(sorted(
                pinfuncs_desc(pin),
                key = lambda s : '' if s.startswith('IO') else s
            ))


        pins = (
            '{:}: {:}'.format(k, pindesc(v))
            for k, v in sorted(pack.pins.items())
        )


        pingriddict = { k : v for k, v in wallpy.viz.list2perimeter(pins) }


        pingriddict[(0, 0)] = '{:}-{:} (VER {:})'.format(
            self.partnum, self.packnum, wallpy.version.get_version()
        )


        G = wallpy.viz.Grid(2, nativedict = pingriddict)
        G.export_tabtext(sys.stdout, default='')


__TL = unittest.TestLoader()

alltests = unittest.TestSuite((
    __TL.loadTestsFromTestCase(Test_xc6slx9),
    __TL.loadTestsFromTestCase(Test_xc3s200a),
))


if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(alltests)

