#!/usr/bin/env python3

import sys
import argparse
import re
import wallpy.version

import devmanager
import schem


if __name__ == '__main__':

    # Commandline argument parser
    parser = argparse.ArgumentParser(
        description='Read and query a device.',
        version=wallpy.version.get_version()
    )

    parser.add_argument(
        '-b', '--bank',
        action='store_true',
        default=False,
        help='Show bank for pins'
    )

    parser.add_argument(
        '-f', '--pinfunc',
        action='store_true',
        default=False,
        help='Show pin functions for pins'
    )

    parser.add_argument(
        'devicename',
        help='Device filename as passed to AddDevice'
    )

    parser.add_argument(
        'packname',
        help='Logical package name within device'
    )



    g_filter = parser.add_mutually_exclusive_group(required=True)

    g_filter.add_argument(
        '--logpins',
        nargs='+',
        help='Logical pins to query',
        metavar='LOGPIN'
    )

    g_filter.add_argument(
        '--re',
        nargs=1,
        help='Show pins matching pinfunc regular expression',
        metavar='REGEXP'
    )


    prefs = parser.parse_args()

    print('Preferences:')
    for k, v in sorted(vars(prefs).items()):
        print('\t', k, ': ', repr(v), sep='')
    print()


    D = devmanager.load_device(prefs.devicename)

    try:
        pack = D.packages[prefs.packname]
    except KeyError:
        print('Unknown package', prefs.packname)
        print('Available packages:', ', '.join(D.packages))
        sys.exit(1)


    if prefs.re:
        R_pat = prefs.re[0]
        R = re.compile(R_pat, flags=re.IGNORECASE)

        prefs.logpins = list()

        for lpk, pin in pack.pins.items():
            for pfname in pin.pinfuncs:
                if R.match(pfname):
                    prefs.logpins.append(lpk)
                    break


    for lpk in prefs.logpins:
        try:
            lpk = int(lpk)
        except ValueError:
            pass

        print(lpk, ':', sep='', end='')

        banks = set()
        pinfuncs = list()

        for pfname, pf in pack.pins[lpk].pinfuncs.items():
            pinfuncs.append(pfname)

            bank = pf.params.get('bank')
            if bank is not None:
                banks.add(bank)

        if prefs.pinfunc:
            print(
                ' PinFuncs: ',
                sorted(pinfuncs, key=schem.name2ana),
                sep='',
                end=''
            )

        if prefs.bank:
            print(
                ' Banks: ',
                sorted(banks),
                sep='',
                end=''
            )

        print()
