#!/usr/bin/env python3

"""Convert part definition file from
    "pinfunc pin(, pin)*" format to
    "pin pinfunc" format.

Read lines from stdin and write to stdout.

Automatically populate power and ground pin functions.
"""

import sys

suffix_type = (
    ('GND', 'pi,<REQUIRED>'),
    ('VCC', 'pi,<REQUIRED>'),
    ('VSS', 'pi,<REQUIRED>'),
    ('VDD', 'pi,<REQUIRED>'),
)

for L in sys.stdin:
    L = L.strip()
    pfname, pinstr = L.split(None, 1)
    pins = (v.strip() for v in pinstr.split(','))

    for pin in pins:
        typestr = ''
        for suffix, v in suffix_type:
            if pfname.lower().endswith(suffix.lower()):
                typestr = '=' + v
                break

        print('{}\t{}{}'.format(pin, pfname, typestr))
