#!/usr/bin/env python3

import sys
import collections

import devmanager
import schem

LINEPREFIX = '#BANKSTANDARDS'

def usage():
    print('Usage:', sys.argv[0], 'bank2class', 'class2standards')
    sys.exit(1)

try:
    fn_b2c, fn_c2s = sys.argv[1:]
except ValueError:
    usage()


class2banks = collections.defaultdict(set)

with open(fn_b2c, 'r') as F:
    for L in F:
        L = L.strip()

        bankstr, classname = L.split()

        classname = classname.lower()

        for b in bankstr.split(','):
            class2banks[classname].add(b)


class2standards = collections.defaultdict(set)

with open(fn_c2s, 'r') as F:
    for L in F:
        L = L.strip().lower()

        classname, standard = L.split()

        # Check standard name
        devmanager.load_iostandard(standard)

        class2standards[classname].add(standard)


for classname, banks in class2banks.items():
    print(
        LINEPREFIX,
        ','.join(sorted(banks, key=schem.name2ana)),
        ','.join(sorted(class2standards[classname])),
        sep='\t'
    )
