#!/usr/bin/env python3

import sys
import os

import wallpy.version

import device
import devimport

VER = wallpy.version.get_version()

PATH = os.path.join('devlib', 'iostandard')


def parse_error(msg, linenum, *extrainfo):
    print('ERROR: {:} on {:}:'.format(msg, linenum), *extrainfo)
    sys.exit(1)


def float_or_none(v):
    try:
        v = float(v)
    except ValueError:
        if v.lower() in {'none', 'n/a'}:
            v = None
        else:
            raise ValueError('Invalid value')

    return v


def gen_file(name, standard):
    filename = os.path.join(PATH, name.lower() + '.py')

    print('CREATING', filename, 'FOR', name)

    with open(filename, 'w') as F:
        print('#!/usr/bin/env python3', file=F)
        print(file=F)
        print('"""I/O standard:', name, file=F)
        print(file=F)
        print('Automatically generated by', sys.argv[0], file=F)
        print(file=F)
        print('Latest tool revision:', VER, file=F)
        print(file=F)
        print('"""', file=F)
        print(file=F)
        print('import wallpy.cleanobject', file=F)
        print('import device', file=F)
        print(file=F)
        print(file=F)

        print('iostandard = ', end='', file=F)
        standard.prettyprint(file=F)

    print()

for linenum, L in enumerate(sys.stdin, start=1):
    try:
        i = L.index('#')
        L = L[:i]
    except ValueError:
        pass

    L = L.strip()

    if not L:
        continue

    words = L.split()
    if len(words) != 3:
        parse_error('Wrong number of words', linenum, len(words))

    name, vcco, vref = words

    try:
        vcco = float_or_none(vcco)
        vref = float_or_none(vref)
    except ValueError:
        parse_error('Invalid voltage', linenum, (vcco, vref))

    standard = device.IOStandard(vcco=vcco, vref=vref)
    gen_file(name, standard)
