#!/usr/bin/env python3

import wallpy
from distutils.core import setup
import distutils.command.build
import distutils.command.sdist
import os
import os.path

try:
    import docutils.core
except ImportError:
    docutils = None

if docutils:
    def convert_rst(infile, outfile, writer_name):
        outfile_name, outfile_ext = os.path.splitext(outfile)
        tmpfile = ''.join((outfile_name, '-gen', outfile_ext))

        pub = docutils.core.Publisher()
        pub.set_components('standalone', 'restructuredtext', writer_name)
        pub.publish(argv = [infile, tmpfile], enable_exit_status = False)

        os.remove(outfile)
        os.rename(tmpfile, outfile)

else:
    def convert_rst(infile, outfile, writer_name):
        print('Not generating', outfile, 'since docutils not available')


class cust_build(distutils.command.build.build):
    def run(self):
        convert_rst('README.txt', 'README.html', 'html')
        distutils.command.build.build.run(self)

class cust_sdist(distutils.command.sdist.sdist):
    def run(self):
        convert_rst('README.txt', 'README.html', 'html')
        distutils.command.sdist.sdist.run(self)

setup(
    version = wallpy.__version__,
    name = "wallpy",
    author = "Daniel P. Seemuth",
    author_email = "wallpy@wallsoftware.com",
    packages = ["wallpy"],
    description = "Collection of loosely-related support modules",
    license = "MIT",
    platforms = ["Independent"],
    url="http://git.wallsoftware.org/gitweb/?p=wallpy.git",
    classifiers = [
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Electronic Design Automation (EDA)",
    ],
    cmdclass = dict(build = cust_build, sdist = cust_sdist)
)
