#!/usr/bin/env python3

######################## BEGIN LICENSE BLOCK ########################
# Copyright (c) 2012 Wall Software, LLC
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use, copy,
# modify, merge, publish, distribute, sublicense, and/or sell copies
# of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
######################### END LICENSE BLOCK #########################

import unittest
import sys

import wallpy.dfs





class Test_DFS(unittest.TestCase):

    def setUp(self):

        self.root = wallpy.dfs.Node(0, (
            wallpy.dfs.Node(1, (
                wallpy.dfs.Node(2, (
                    wallpy.dfs.Node(3),
                    wallpy.dfs.Node(4),
                    wallpy.dfs.Node(5),
                )),
                wallpy.dfs.Node(6),
                wallpy.dfs.Node(7, (
                    wallpy.dfs.Node(8),
                    wallpy.dfs.Node(9),
                )),
            )),
            wallpy.dfs.Node(10),
            wallpy.dfs.Node(11, (
                wallpy.dfs.Node(12),
            )),
        ))


        node4 = self.root.children[0].children[0].children[1]
        node10 = self.root.children[1]
        node12 = self.root.children[2].children[0]

        self.assertEqual(node4.val, 4, 'incorrect node')
        self.assertEqual(node10.val, 10, 'incorrect node')
        self.assertEqual(node12.val, 12, 'incorrect node')


        node10.children.append(node12)
        node4.children.append(self.root)


        self.expect_dfs = [3, 4, 5, 2, 6, 8, 9, 7, 1, 12, 10, 11, 0]


    def test_dfs_recursive(self):

        def callback(node, destlist):
            destlist.append(node.val)


        resultlist = list()

        self.root.dfs_recursive(callback = callback,
                                callback_posargs = [resultlist])

        self.assertListEqual(resultlist, self.expect_dfs,
                             'invalid dfs recursive results')


    def test_dfs_iterative(self):

        def callback(node, destlist):
            destlist.append(node.val)


        resultlist = list()

        self.root.dfs_iterative(callback = callback,
                                callback_posargs = [resultlist])

        self.assertListEqual(resultlist, self.expect_dfs,
                             'invalid dfs iterative results')




__TL = unittest.TestLoader()

alltests = unittest.TestSuite((
    __TL.loadTestsFromTestCase(Test_DFS),
))


if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(alltests)
