#!/usr/bin/env python3

######################## BEGIN LICENSE BLOCK ########################
# Copyright (c) 2012 Wall Software, LLC
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use, copy,
# modify, merge, publish, distribute, sublicense, and/or sell copies
# of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
######################### END LICENSE BLOCK #########################

"""Pretty printing utility, support classes.

Exported Classes:

PrettyPrinter -- keep track of indent level; offer print function
InitialSeparator -- infinite iterator with initial, successive values

"""


class PrettyPrinter(object):
    def __init__(self, numspaces, file, usetabs = False):
        self.numspaces = int(numspaces)
        self.file = file
        self.usetabs = usetabs


    def indenter(self, numspaces):
        if self.usetabs:
            numtabs = numspaces // 8
            numspaces = numspaces % 8

            return ''.join(['\t' * numtabs, ' ' * numspaces])

        return ' ' * numspaces




    def print(self, *args, extraspaces = 0):

        print(
            self.indenter(self.numspaces + extraspaces),
            *args,
            sep = '',
            file = self.file)




class InitialSeparator(object):
    """Generate an iterator returning a sequence sep0, sep1, sep1, ..."""

    def __init__(self, sep0, sep1):
        self.pastfirst = False
        self.sep0 = sep0
        self.sep1 = sep1

    def __iter__(self):
        return self

    def __next__(self):
        if self.pastfirst:
            ret = self.sep1
        else:
            ret = self.sep0
            self.pastfirst = True

        return ret
